<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;
use Util\Repositories;
use Util\NovoCodigo;

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil };
use Ems\Repository\{ PeriodoRepository, SituacaoFuncionalRepository };

class SituacaoFuncionalService {
    /** @var Environment */
    protected $environment;

    /** @var EntityManager */
    protected $entityManager;

    /** @var SituacaoFuncionalRepository */
    protected $situacaoFuncionalRepository;

    /** @var PeriodoRepository */
    protected $periodoRepository;

    public function __construct(
        EntityManager                   $entityManager,
        Environment                     $environment,
        SituacaoFuncionalRepository     $situacaoFuncionalRepository,
        PeriodoRepository               $periodoRepository
    ) {
        $this->entityManager                = $entityManager;
        $this->environment                  = $environment;
        $this->situacaoFuncionalRepository  = $situacaoFuncionalRepository;
        $this->periodoRepository            = $periodoRepository;
    }

  /**
    * Rotina responsável por obter a última situação funcional de um determinado vínculo em um período de apuração ou dentro da competência. Caso
    * seja informado um período somente, a data de competência é obtida a partir dele
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaUltimaSituacaoFuncional
    */
    public function retornaUltimaSituacaoFuncional(
        int     $nrVinculom,
        string  $nrPeriodoApuracao = null,
        string  $mesCompetencia = null
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        if (empty($nrPeriodoApuracao) && empty($mesCompetencia)) {
            return false;
        }

        $dtCompetenciaSituacaoFuncional = $mesCompetencia;
        if (empty($mesCompetencia)) {
            $dtCompetenciaSituacaoFuncional = $this->periodoRepository->getDtmescompetencByPeriodo($nrOrg, $nrPeriodoApuracao);
        }

        return $this->situacaoFuncionalRepository->retornaUltimaSituacaoFuncional(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom,
            $dtCompetenciaSituacaoFuncional    
        );
    }
    
  /**
    * Rotina responsável por obter os períodos de estabilidade de um determinado vínculo
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaPeriodosEstabilidade
    */
    public function retornaPeriodosEstabilidade(
        int     $nrVinculom
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        return $this->situacaoFuncionalRepository->retornaPeriodosEstabilidade(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom
        );
    }

    public function setAlteEscala(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRALTEESCALA'] =
            $row['NRALTEESCALA'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTEESCALA', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRVINCULOM']           = (int) $row['NRVINCULOM'];
        $params['DTINIESCALA']          = (string) $row['DTINIESCALA'];
        $params['DTFIMESCALA']          = $row['DTFIMESCALA'] ?? null;
        $params['NRESCALATRABM']        = (int) $row['NRESCALATRABM'];
        $params['NRHORDIAM']            = $row['NRHORDIAM'] ?? null;
        $params['NRMOTIVOESCALA']       = $row['NRMOTIVOESCALA'] ?? null;
        $params['DSOBSERVACAO']         = $row['DSOBSERVACAO'] ?? null;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;
        $params['NRTURNO']              = $row['NRTURNO'] ?? null;
        $params['QTHRLIMITESEPHEXTRA']  = $row['QTHRLIMITESEPHEXTRA'] ?? null;

        $this->situacaoFuncionalRepository->setAlteEscala($params);

        return $params['NRALTEESCALA'];
    }

    public function setAlteSituacaoFuncional(
        array $row
    ): int {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperador = (string) $this->environment->getCdOperador();
        $params = [];

        $params['NRALTESITUFUNC'] =
            $row['NRALTESITUFUNC'] ??
            NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTESITUFUNC', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRMOTIVOAFASTA']       = $row['NRMOTIVOAFASTA'] ?? null;
        $params['NRPROXSITUFUNC']       = $row['NRPROXSITUFUNC'] ?? null;
        $params['CDTABECDI']            = $row['CDTABECDI'] ?? null;
        $params['CDDIAGNOST']           = $row['CDDIAGNOST'] ?? null;
        $params['NRSITUFUNCM']          = (int) $row['NRSITUFUNCM'];
        $params['NRVINCULOM']           = (int) $row['NRVINCULOM'];
        $params['DTINISITUFUNC']        = (string) $row['DTINISITUFUNC'];
        $params['DTFIMSITUFUNC']        = $row['DTFIMSITUFUNC'] ?? null;
        $params['DTVERIFSITUFUNC']      = $row['DTVERIFSITUFUNC'] ?? null;
        $params['DTPREVIRETOSITU']      = $row['DTPREVIRETOSITU'] ?? null;
        $params['DTPRORRSITUFUNC']      = $row['DTPRORRSITUFUNC'] ?? null;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;
        $params['DSOBSALTSITFUNC']      = $row['DSOBSALTSITFUNC'] ?? null;
        $params['NRQTDIASITUFUNC']      = $row['NRQTDIASITUFUNC'] ?? null;
        $params['DTESTABSITUFUNC']      = $row['DTESTABSITUFUNC'] ?? null;
        $params['NRPESSOA']             = $row['NRPESSOA'] ?? null;
        $params['QTDIASATESTADOEMPR']   = $row['QTDIASATESTADOEMPR'] ?? null;

        $this->situacaoFuncionalRepository->setAlteSituacaoFuncional($params);

        return $params['NRALTESITUFUNC'];
    }
}
